# Copyright (c) 2008-2009 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: interfaces.py 16223 2009-06-16 12:55:12Z ctheune $

import re

import grok
import zc.sourcefactory.basic
import zope.interface
import zope.schema
import zope.component


class IDirectory(zope.interface.Interface):
    """The directory."""

    paltry = zope.schema.ASCIILine(
        title=u'Location of paltry directory')


class ITitled(zope.interface.Interface):
    """Entity with a title."""

    title = zope.schema.TextLine(title=u'Title')


class ITitle(zope.interface.Interface):
    """Title of an object."""

    def __str__():
        """Returns the title."""


@grok.implementer(ITitle)
@grok.adapter(ITitled)
def title(context):
    return context.title


class UserAccounts(zc.sourcefactory.basic.BasicSourceFactory):
    """List all unix accounts that belong to a person."""

    def getValues(self):
        unixaccounts = zope.component.getUtility(IUnixAccounts)
        return tuple(account for account in unixaccounts.sorted_values()
                     if IUserAccount.providedBy(account))

    def getTitle(self, value):
        return ITitle(value)


class IPersons(zope.interface.Interface):
    """A collection of persons."""


class IPerson(zope.interface.Interface):
    """A Person."""

    first_name = zope.schema.TextLine(title=u'First name')
    last_name = zope.schema.TextLine(title=u'Last name')
    unixaccount = zope.schema.Choice(
        title=u'User account', source=UserAccounts(),
        constraint=lambda account: (account.owner is None or
                                    account.owner.unixaccount is account),
        required=False,
        readonly=True)


class ICustomers(zope.interface.Interface):
    """Collection of the customers."""


class Persons(zc.sourcefactory.basic.BasicSourceFactory):
    """List a available persons."""

    def getValues(self):
        persons = zope.component.getUtility(IPersons)
        return tuple(persons.sorted_values())

    def getTitle(self, value):
        return ITitle(value)


class ICustomer(zope.interface.Interface):
    """A Customer."""

    id = zope.schema.TextLine(
        title=u'Customer id',
        readonly=True)

    title = zope.schema.TextLine(
        title=u'Title',
        readonly=True)

    persons = zope.schema.Set(
        title=u'Persons',
        value_type=zope.schema.Choice(
            title=u'Person',
            source=Persons()))


class IResourceGroups(zope.interface.Interface):
    """A collection of resource groups."""


class IResourceGroup(ITitled):
    """A group of resources."""

    group = zope.schema.ASCIILine(
        title=u'unix user group name',
        min_length=2,
        max_length=8,
        constraint=re.compile(r'[a-z0-9_-]+$').match)
    gid = zope.schema.Int(title=u'unix user group id',
                          min=1001, # 1000 is reserved for the default group
                          max=64999,
                          readonly=True)
    persons = zope.schema.Set(
        title=u'Persons associated with this group',
        value_type=zope.schema.Choice(title=u'Person',
                                      source=Persons()))


class UnixAccountOwners(Persons):

    def getValues(self):
        owners = list(super(UnixAccountOwners, self).getValues())
        customers = zope.component.getUtility(ICustomers).sorted_values()
        for customer in customers:
            resourcegroups = zope.component.getUtility(
                IResourceGroups, context=customer)
            owners.extend(resourcegroups.sorted_values())
        return tuple(owners)


class IUnixAccounts(zope.interface.Interface):
    """A collection of unix accounts."""


class IUnixAccount(zope.interface.Interface):
    """Representation of a unix account."""

    owner = zope.schema.Choice(
        title=u'Owner', source=UnixAccountOwners(), readonly=True,
        constraint=lambda owner: owner.unixaccount.owner is owner)
    username = zope.schema.ASCIILine(
        title=u'Unix login',
        readonly=True,
        min_length=2,
        max_length=8,
        constraint=re.compile(r'[a-z0-9_-]+$').match)
    uid = zope.schema.Int(title=u'Unix user id',
                          min=1000,
                          max=64999,
                          readonly=True)


class IUserAccount(IUnixAccount):
    """Representation of a unix account that belongs to a person."""

    password = zope.schema.Password(title=u'Password')
    password2 = zope.schema.Password(title=u'Repeat the password')
    crypted_password = zope.interface.Attribute(
        u'libc6 compatible md5 hash of the password')

    @zope.interface.invariant
    def arePasswordsEqual(user):
        if user.password != user.password2:
            raise zope.interface.Invalid(
                u"""Passwords are not equal.""")


class IGECOS(zope.interface.Interface):
    """Structured GECOS information of a unix account."""

    fullname = zope.schema.TextLine(title=u'Full name',
                                    readonly=True)


class IStorages(zope.interface.Interface):
    """A collection of storages."""


class IStorage(ITitled):
    """A storage."""

    size = zope.schema.Int(title=u'size in GByte')


class IIPSubnets(zope.interface.Interface):
    """A collection of IP subnets."""


class IIPSubnet(zope.interface.Interface):
    """An IP subnet."""

    netmask = zope.schema.TextLine(title=u'netmask')


class IPhysicalHosts(zope.interface.Interface):
    """A collection of pyhsical hosts."""


class IPhysicalHost(ITitled):
    """A pyhsical host."""


class IVirtualHosts(zope.interface.Interface):
    """A collection of virtual hosts."""


class IVirtualHost(ITitled):
    """A virtual host."""


class IFQDNs(zope.interface.Interface):
    """A collection of fully qualified domain names."""


class IFQDN(zope.interface.Interface):
    """A fully qualified domain name."""

    fqdn = zope.schema.TextLine(title=u'fully qualified domain name')


class IDomains(zope.interface.Interface):
    """A collection of domains."""


class IDomain(ITitled):
    """A domain."""


class IPOSIXIds(zope.interface.Interface):
    """A generator for user/group id which fit in a configurable range."""

    def register(object):
        """Return an integer id."""
